0.5::second => dur beat;
0.5::second => dur beatHalf;
1.0 => float counter; //counter of each run so that we can do changes depending on how many times it was run
3.5 => float gainMult; //gain multiplier

SinOsc hb => ADSR env => NRev rev => dac;
SinOsc backing => ADSR env2 => NRev rev2 => dac;
SinOsc mel => ADSR env3 => NRev rev3 => dac;
PulseOsc beep => dac;
SndBuf thunder => dac; //sound buffer ugen lets us playback from file system

//give main.ck a chance to run/set up
0 => hb.gain => backing.gain => mel.gain => beep.gain;
15::second => now;

//play beeps to show how visualizer responds
for (1 => int i; i < 13; i++) {
    0.05 => beep.gain;
    Std.mtof(i * 8) => beep.freq;
    0.35::second => now;
    0 => beep.gain;
    1::second => now;
}

//play thunder to set the mood
0.8 => thunder.gain;
me.dir() + "thunder.wav" => thunder.read;
10::second => now;

(1::ms, beat/2, 0, 1::ms) => env.set;
(1::ms, beat/2, 0, 1::ms) => env2.set;
(1.5::ms, beat, 0.0, 10::second) => env3.set;
0.3 => rev.mix;
0.0 => rev3.mix;

//notes and note durations for each part
[
    56, 60, 63, 60,
    58, 61, 65, 61,
    56, 60, 63, 60,
    54, 58, 61, 58,
] @=> int backingNotes[]; //giselle backing

[
    0, 56, 0, 56, 49, 0, 48, 51, 49,
    0, 56, 56, 49, 48, 49, 53, 51, 51, 49,
    0, 56, 58, 56, 49, 49, 51, 53, 53, 54,
    56, 54, 53, 53, 51, 49, 46,
    48, 49, 51, 53, 56, 51
] @=> int melody[];

[
    2.0, 2.0, 2.0, 1, 1, 5.0, 1, 1, 1,
    2, 2, 1, 2, 2, 3, 1, 0.5, 0.5, 2,
    2, 2, 1, 2, 1, 1, 1, 1, 0.5, 0.5,
    1, 0.5, 0.5, 1, 0.5, 0.5, 2,
    1, 2, 2, 1, 3, 5
] @=> float melodyDurations[];

//plays the melody repeatedly
fun void playMelody() {
    for (int i; i < melody.size(); i++) {
        .18 * counter => mel.gain;
        2 => env3.keyOn;
        Std.mtof(melody[i]) => mel.freq;
        melodyDurations[i] * beatHalf => now;
        0 => mel.gain;
    }
}

//play the heartbeat in the background
fun void playHB(int range) {
    for (int i; i < range; i++) {
        //first heartbeat
        Std.mtof(30) => hb.freq;
        .35 * gainMult => hb.gain; //amplitude
        1 => env.keyOn;
        beatHalf => now;
        .25 * gainMult => hb.gain; //amplitude
        Std.mtof(29) => hb.freq;
        2 => env.keyOn;
        beat => now;

        //pause before second heartbeat
        0 => hb.gain;
        6.5::second => now;

        //second heartbeat
        Std.mtof(30) => hb.freq;
        .35 * gainMult => hb.gain; //amplitude
        1 => env.keyOn;
        beatHalf => now;
        .25 * gainMult => hb.gain; //amplitude
        Std.mtof(29) => hb.freq;
        10 => env.keyOn;
        beat * 3 => now;

        //silence so that the next for loop run doesn't play immediately after
        0 => hb.gain;
        4.5::second => now;
    }
}

//play the backing notes. make it more and more clear as each loop occurs and also softer.
fun void playBacking() {
    1 / counter => rev2.mix;
    Math.exp(counter * -1)  => backing.gain;
    for (int j; j < 4; j++) {
        for (int i; i < backingNotes.size(); i++) {
            Std.mtof(backingNotes[i]) => backing.freq;
            1 => env2.keyOn;
            beatHalf => now;
        }
    }
}

//play heartbeat twice to set the mood
playHB(2);

//play it all together! the main loop.
while (true) {
    spork ~ playHB(4);
    spork ~ playBacking();
    playMelody();
    counter + 1.0 => counter;
}