public class Sun extends GGen {
    16 => int NUM_RAYS;
    10 => global float DISPLAY_WIDTH; // width of waveform and spectrum display
    1024 => global int WINDOW_SIZE; // window size
    GPoints sunraysBig[NUM_RAYS];

    //create and position the sun
    GCircle sun;
    sun --> GG.scene();
    sun.sca( @(1, 1, 1) );
    sun.pos( @(3, 3, 2) );

    //texture it!
    FlatMaterial matSun;
    FileTexture bgSun;
    bgSun.path("sun.png");
    bgSun.wrap(1, 1);
    matSun.diffuseMap(bgSun);
    sun.mat(matSun);


    // waveform renderer for time domain as rays; implement all sun rays
    [.1, .05, .3, .45] @=> float raySize[]; // /4 logic
    [    @(1.5, 0, 0), @(-1.5, 0, 0), @(0, 1.5, 0), @(0, -1.5, 0),    @(-1, 1, 0), @(1, 1, 0), @(-1, -1, 0), @(1, -1, 0),    @(0, 0, 0), @(0, 0, 0), @(0, 0, 0), @(0, 0, 0),    @(0, 0, 0), @(0, 0, 0), @(0, 0, 0), @(0, 0, 0)
    ] @=> vec3 posRayBig[]; //positions of the sun rays
    [0.0, 0.0, Math.PI / 2, Math.PI / 2,    Math.PI / 4, -1 * Math.PI / 4, -5 * Math.PI / 4, -7 * Math.PI / 4,    Math.PI / 3, 5 * Math.PI / 6, 4 * Math.PI / 3, 11 * Math.PI / 6,    Math.PI / 6, 2 * Math.PI / 3, 7* Math.PI / 6, 5 * Math.PI / 3
    ] @=> float rayRot[]; //rotations of the sun rays
    for (0 => int i; i < NUM_RAYS; i++) {
        sunraysBig[i] @=> auto r;
        r --> sun;
        r.mat().color(@(1, 0.847, 0));
        r.sca(raySize[i / 4]);
        r.rotZ(rayRot[i]);
        r.pos(posRayBig[i]);
        r.posZ(-.001);
    }

    //from soundpeek
    adc => Gain input; // which input?
    input => Flip accum => blackhole; // accumulate samples from mic
    input => PoleZero dcbloke => FFT fft => blackhole; // take the FFT
    .95 => dcbloke.blockZero; // set DC blocker
    WINDOW_SIZE => accum.size; // set size of flip
    Windowing.hann(WINDOW_SIZE) => fft.window; // set window type and size
    WINDOW_SIZE*2 => fft.size; // set FFT size (will automatically zero pad)
    Windowing.hann(WINDOW_SIZE) @=> float window[]; // get a reference for our window for visual tapering of the waveform
    float samples[0]; // sample array
    complex response[0]; // FFT response
    vec3 positions[WINDOW_SIZE]; // a vector to hold positions
    vec3 positionsCircle[WINDOW_SIZE]; // a vector to hold positions

    //from soundpeek
    fun void map2waveformLine( float in[], vec3 out[] ) {
        if( in.size() != out.size() )    {
            <<< "size mismatch in map2waveform()", "" >>>;
            return;
        }
        // mapping to xyz coordinate
        int i;
        DISPLAY_WIDTH => float width;
        for( auto s : in )    {
            -width/2 + width/WINDOW_SIZE*i => out[i].x; // space evenly in X
            s*6 * window[i] => out[i].y; // map y, using window function to taper the ends
            0 => out[i].z; // a constant Z of 0
            i++; // increment
        }
    }

    //from soundpeek
    fun void doAudio() {
        while( true ) {
            // upchuck to process accum
            accum.upchuck();
            // get the last window size samples (waveform)
            accum.output( samples );
            // upchuck to take FFT, get magnitude reposne
            fft.upchuck();
            // get spectrum (as complex values)
            fft.spectrum( response );
            // jump by samples
            WINDOW_SIZE::samp/2 => now;
        }
    }
    spork ~ doAudio();

    fun void update ( float dt ) {
        sun.rotZ(0.01); //make it spin!
        map2waveformLine( samples, positions ); // map to interleaved format
        for (auto r: sunraysBig) r.geo().positions(positions); //update for each sun ray
    }
}